/*****************************************************************************
* wanpipe_time.h WANPIPE(tm) - Portable Time routines for Sangoma MS Windows API.
*
* Author: 	David Rokhvarg <davidr@sangoma.com
*
* Copyright:	(c) 2007 Sangoma Technologies Inc.
*
* ============================================================================
* Nov 27, 2008  David Rokhvarg	Initial version
* ============================================================================
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
*/

#ifndef _WANPIPE_TIME_H
#define _WANPIPE_TIME_H

#if !defined(WAN_KERNEL)

#include <time.h>		/* time_t, localtime() */
#include <Winsock.h>	/* struct timeval */

static void gettimeofday(struct timeval *tv, void*not_used)
{	
	time( (time_t*)&tv->tv_sec ); /* Get time as long integer. Exported by  */
}

static void localtime_r(time_t *long_time, struct tm *t)
{
	memcpy(t, localtime(long_time), sizeof(struct tm)); /* Convert to local time. */
}

static void usleep(int microseconds)
{
	/* translate microseconds into milliseconds and call Sleep() */
	Sleep(microseconds / 1000);
}

/* UNIX sleep - Sleep for the specified number of seconds */
static void sleep(int seconds)
{
	/* translate seconds into milliseconds and call Sleep() */
	Sleep(seconds * 1000);
}

#if 0
#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS)
#define DELTA_EPOCH_IN_MICROSECS  11644473600000000Ui64
#else
#define DELTA_EPOCH_IN_MICROSECS  11644473600000000ULL
#endif

struct timezone 
{
	int  tz_minuteswest; /* minutes W of Greenwich */
	int  tz_dsttime;     /* type of dst correction */
};

static int gettimeofday(struct timeval *tv, struct timezone *tz)
{
	FILETIME ft;
	unsigned __int64 tmpres = 0;
	static int tzflag;

	if (NULL != tv)
	{
		GetSystemTimeAsFileTime(&ft);

		tmpres |= ft.dwHighDateTime;
		tmpres <<= 32;
		tmpres |= ft.dwLowDateTime;

		/*converting file time to unix epoch*/
		tmpres /= 10;  /*convert into microseconds*/
		tmpres -= DELTA_EPOCH_IN_MICROSECS; 
		tv->tv_sec = (long)(tmpres / 1000000UL);
		tv->tv_usec = (long)(tmpres % 1000000UL);
	}

	if (NULL != tz)
	{
		if (!tzflag)
		{
			_tzset();
			tzflag++;
		}
		tz->tz_minuteswest = _timezone / 60;
		tz->tz_dsttime = _daylight;
	}

	return 0;
}
#endif

#endif/* WAN_KERNEL */
#endif/* _WANPIPE_TIME_H */
